/********************************************
Personal library of Cyril Roussillon
http://crteknologies.free.fr
cyril42e@gmail.com
2006-2007
*********************************************/

#include "cyril-imgproc.h"

// void __() {}

void decime(uchar *src, int src_w, int src_h, int src_step,
						uchar *dst, int dst_w, int dst_h, int dst_step, int spix)
{
	int i, j, k, ii, jj;
	int I, J, lastI, lastJ;

	int line, pix, area;
	int maxvals[3]; ASSERT(spix <= 3);
	
	real coefx = src_w/(real)dst_w; 
	real coefy = src_h/(real)dst_h; 
	ASSERT(coefx >= 1); ASSERT(coefy >= 1);

	lastI=-1;
	for(i = 0; i < dst_h; i++) 
	{
		I = (i+1)*coefy;
		lastJ=-1;
		for(j = 0; j < dst_w; j++)
		{
			J = (j+1)*coefx;
			for(k = 0; k < spix; k++) maxvals[k] = 0;
			for(ii = lastI+1; ii <= I; ii++) 
			{
				line = ii*src_step;
				for(jj = lastJ+1; jj <= J; jj++)
				{ 
					pix = line + spix*jj;
					for(k = 0; k < spix; k++) if (src[pix+k] > maxvals[k]) maxvals[k] = src[pix+k];
				}
			}
			area = (I-lastI)*(J-lastJ); ASSERT(area);
			for(k = 0; k < spix; k++) dst[i*dst_step+j*spix + k] = maxvals[k];
			lastJ = J;
		}
		lastI = I;
	}
}


void resize(uchar *src, int src_w, int src_h, int src_step,
						uchar *dst, int dst_w, int dst_h, int dst_step, int spix)
{
	int i, j, k, ii, jj;
	int I, J, lastI, lastJ;
	real If, Jf, lastIf, lastJf;
	real weightI, weightJ;
	real area;

	int line, pix;
	int sum[3];
	
	real coefx = src_w/(real)dst_w; 
	real coefy = src_h/(real)dst_h; 
	ASSERT(coefx >= 1); ASSERT(coefy >= 1);

	lastI=0; lastIf = 1;
	for(i = 0; i < dst_h; i++) 
	{
		If = (i+1)*coefy;// + (coefy-1);
		I = (int)If; If = If-(real)I;
		if (If < 0.01) { I--; If = 1; } // assurance
		lastJ=0; lastJf = 1;
		for(j = 0; j < dst_w; j++)
		{
			Jf = (j+1)*coefx;// + (coefx-1);
			J = (int)Jf; Jf = Jf-(real)J;
			if (Jf < 0.01) { J--; Jf = 1; }
			sum[0] = 0; sum[1] = 0; sum[2] = 0;
			for(ii = lastI; ii <= I; ii++) 
			{
				line = ii*src_step;
				weightI = (ii == lastI ? lastIf : (ii == I ? If : 1));
				for(jj = lastJ; jj <= J; jj++)
				{ 
					pix = line + spix*jj;
					weightJ = (jj == lastJ ? lastJf : (jj == J ? Jf : 1));
					for(k = 0; k < spix; k++) sum[k] += (int)(weightI*weightJ*src[pix+k]);
				}
			}
			area = ((I+If)-(lastI+1-lastIf))*((J+Jf)-(lastJ+1-lastJf)); 
			if (area == 0) printf("I %d, If %g, lastI %d, lastIf %g, J %d, Jf %g, lastJ %d, lastJf %g (coefx %g, coefy %g)\n",
				I, If, lastI, lastIf, J, Jf, lastJ, lastJf, coefx, coefy);
			ASSERT(area != 0);
			for(k = 0; k < spix; k++) dst[i*dst_step+j*spix + k] = (uchar)(sum[k] / area);
			lastJ = J; lastJf = 1-Jf;
		}
		lastI = I; lastIf = 1-If;
	}
}

