/********************************************
Personal library of Cyril Roussillon
http://crteknologies.free.fr
cyril42e@gmail.com
2006-2007
*********************************************/

//////////////////////////////////////////////////////////////////
// messages part1
//////////////////////////////////////////////////////////////////
/*
	Here are the message macro definitions, at the beginning, before the non multiple include ifdef,
	so that you can change the value of one activation define and include again this lib in order to locally
	enable or disable messages
*/

/*
	FERROR : Fatal Error (red), I can't do anything so I exit the program
	ERROR : Error (red), something cannot be done (should be followed by a return / break / continue)
	WARN : Warning (yellow), something doesn't seem normal but I will continue as if it was
	INFO : Information (blue), some information purpose stuff (everything's normal)
	DEBUG : Debug (green), some specific printings to debug a part of the program
	CHECK : Checkpoint (gray), eg to try to find where the program stops
*/

#ifdef __cplusplus
// LANGUAGE C++

#undef ERROR
#undef FERROR
#ifndef _NO_ERROR_MSG_
	#define FERROR(msg) { std::cerr << clBRED <<    "ERROR:" << clRED    << __FILE__ << ":" << __LINE__ << clBRED    << "# " << msg << clNORMAL << std::endl; if (!STOP_PROGRAM) abort(); }
	#define ERROR(msg) std::cerr << clBRED <<    "ERROR:" << clRED    << __FILE__ << ":" << __LINE__ << clBRED    << "# " << msg << clNORMAL << std::endl;
#else
	#define FERROR(msg)
	#define ERROR(msg)
#endif

#undef WARN
#ifndef _NO_WARN_MSG_
	#define  WARN(msg) std::cerr << clBYELLOW << " WARN:" << clYELLOW << __FILE__ << ":" << __LINE__ << clBYELLOW << "# " << msg << clNORMAL << std::endl;
#else
	#define WARN(msg)
#endif

#undef INFO
#ifndef _NO_INFO_MSG_
	#define  INFO(msg) std::cout << clBBLUE <<   " INFO:" << clBLUE   << __FILE__ << ":" << __LINE__ << clBBLUE   << "# " << msg << clNORMAL << std::endl;
#else
	#define INFO(msg)
#endif

#undef CHECK
#ifndef _NO_DEBUG_MSG_
	#define DEBUG(msg) std::cout << clBGREEN <<  "DEBUG:" << clGREEN  << __FILE__ << ":" << __LINE__ << clBGREEN  << "# " << msg << clNORMAL << std::endl;
#else
	#define DEBUG(msg)
#endif

#ifndef _NO_CHECK_MSG_
	#define CHECK      std::cout << clBGRAY <<   "CHKPT:" << clGRAY   << __FILE__ << ":" << __LINE__                             << clNORMAL  << std::endl;
#else
	#define CHECK
#endif

#undef BREAK
#ifndef _NO_BREAK_MSG_
//	#define BREAK(msg, ...) { fprintf(stderr, clBGRAY "BRKPT:" clBGRAY "%s:%04d" clBGRAY "# ", __FILE__, __LINE__); fprintf(stderr, msg,  ## __VA_ARGS__); fprintf(stderr, clNORMAL "\n"); getchar(); }
	#define BREAK      { std::cerr << clBGRAY << "BRKPT:" << clBGRAY << __FILE__ << ":" << __LINE__ << " # gdb " << /*argv[0]*/ "snake" << " " << getpid() << std::endl; getchar(); }
#else
	#define BREAK
#endif


#undef ASSERT
#undef ASSERTM
#undef ASSERTC
#undef ASSERTCM
#ifndef _NO_ASSERT_
	#define ASSERT(test) if ((test) == 0) FERROR("ASSERT \"" << #test << "\" failed");
	#define ASSERTM(test,msg) if ((test) == 0) FERROR("ASSERT failed:" << msg);
	#define ASSERTC(test,comp,val) { double test_=(test), val_=(val); if (!(test_ comp val_)) \
		FERROR("ASSERT \"" << #test << "\" failed with code " << test << " instead of " << #comp << #val); }
	#define ASSERTCM(test,comp,val,msg) { double test_=(test), val_=(val); if (!(test_ comp val_)) \
		FERROR("ASSERT \"" << #test << "\" failed with code " << test << " instead of " << #comp << #val << " ; " << msg); }
#else
	#define ASSERT(test)
	#define ASSERTM(test,msg,...)
	#define ASSERTC(test,comp,val)
	#define ASSERTCM(test,comp,val,msg,...)
#endif

#else
// LANGUAGE C

#undef ERROR
#undef FERROR
#ifndef _NO_ERROR_MSG_
	#define ERROR(msg, ...) { teeprintf(clBRED "ERROR:" clRED "%s:%04d" clBRED "# ", __FILE__, __LINE__); teeprintf(msg,  ## __VA_ARGS__); teeprintf(clNORMAL "\n"); if (LOG_FILE) fflush(LOG_FILE); }
	#define FERROR(msg, ...) { teeprintf(clBRED "FATAL ERROR:" clRED "%s:%04d" clBRED "# ", __FILE__, __LINE__); teeprintf(msg,  ## __VA_ARGS__); teeprintf(clNORMAL "\n"); if (LOG_FILE) fflush(LOG_FILE); if (!STOP_PROGRAM) abort(); }
#else
	#define ERROR(msg, ...)
	#define FERROR(msg, ...)
#endif

#undef WARN
#ifndef _NO_WARN_MSG_
	#define WARN(msg, ...) { teeprintf(clBYELLOW " WARN:" clYELLOW "%s:%04d" clBYELLOW "# ", __FILE__, __LINE__); teeprintf(msg,  ## __VA_ARGS__); teeprintf(clNORMAL "\n"); }
#else
	#define WARN(msg, ...)
#endif

#undef INFO
#undef INFO2
#ifndef _NO_INFO_MSG_
	#define INFO(msg, ...) { teeprintf(clBBLUE "INFO+:" clBLUE "%s:%04d" clBBLUE "%s# %s", __FILE__, __LINE__, FILL_BLANK(FN_SIZE-(int)strlen(__FILE__)), FILL_BLANK(INFO_INDENT*2)); teeprintf(msg,  ## __VA_ARGS__); teeprintf(clNORMAL "\n"); }
	#define INFO2(msg, ...) { teeprintf(clBCYAN "INFO+:" clCYAN "%s:%04d" clBCYAN "%s# %s", __FILE__, __LINE__, FILL_BLANK(FN_SIZE-(int)strlen(__FILE__)), FILL_BLANK(INFO_INDENT*2)); teeprintf(msg,  ## __VA_ARGS__); teeprintf(clNORMAL "\n"); }
#else
	#define INFO(msg, ...)
	#define INFO2(msg, ...)
#endif

#undef DEBUG
#ifndef _NO_DEBUG_MSG_
	#define DEBUG(msg, ...) { teeprintf(clBGREEN "DEBUG:" clGREEN "%s:%04d" clBGREEN "# ", __FILE__, __LINE__); teeprintf(msg,  ## __VA_ARGS__); teeprintf(clNORMAL "\n"); }
#else
	#define DEBUG(msg, ...)
#endif

#undef CHECK
#ifndef _NO_CHECK_MSG_
	#define CHECK      { fprintf(stderr, clBGRAY "CHKPT:" clBGRAY "%s:%04d" clNORMAL "\n", __FILE__, __LINE__); }
#else
	#define CHECK
#endif

#undef BREAK
#ifndef _NO_BREAK_MSG_
//	#define BREAK(msg, ...) { fprintf(stderr, clBGRAY "BRKPT:" clBGRAY "%s:%04d" clBGRAY "# ", __FILE__, __LINE__); fprintf(stderr, msg,  ## __VA_ARGS__); fprintf(stderr, clNORMAL "\n"); getchar(); }
	#define BREAK      { fprintf(stderr, clBGRAY "BRKPT:" clBGRAY "%s:%04d # gdb %s %d" clNORMAL "\n", __FILE__, __LINE__, /*argv[0]*/ "violajones", getpid()); getchar(); }
#else
	#define BREAK
#endif


#undef LOG
#ifndef _NO_LOG_
	#define LOG(msg, ...) { teeprintf(msg, ## __VA_ARGS__); }
#else
	#define LOG(msg, ...) 
#endif

#undef ASSERT
#undef ASSERTM
#undef ASSERTC
#undef ASSERTCM
#ifndef _NO_ASSERT_
	#define ASSERT(test) if ((test) == 0) { char msg2_[300]="ASSERT \""; strncat(msg2_,#test,strcspn(#test,"(")); strncat(msg2_,"\" failed",25); FERROR(msg2_); }
	#define ASSERTM(test,msg,...) if ((test) == 0) { char msg_[300]="ASSERT failed : "; strncat(msg_,msg,250); FERROR(msg_, ## __VA_ARGS__); }
	#define ASSERTC(test,comp,val) { double test_=(test), val_=(val); if (!(test_ comp val_)) { char msg2_[300]="ASSERT \""; \
        strncat(msg2_,#test,strcspn(#test,"(")); strncat(msg2_,"\" failed with code %.9g instead of ",100); \
        strncat(msg2_,#comp,100); strncat(msg2_," %.9g / ",100); strncat(msg2_,#val,100); FERROR(msg2_,test_,val_); }}
	#define ASSERTCM(test,comp,val,msg,...) { double test_=(test), val_=(val); if (!(test_ comp val_)) { char msg2_[300]="ASSERT \""; \
        strncat(msg2_,#test,strcspn(#test,"(")); strncat(msg2_,"\" failed with code %.9g instead of ",100); \
        strncat(msg2_,#comp,100); strncat(msg2_," %.9g / ",100); strncat(msg2_,#val,100); strncat(msg2_," ; ",100); strncat(msg2_,msg,100); FERROR(msg2_,test_,val_, ## __VA_ARGS__); }}
#else
	#define ASSERT(test)
	#define ASSERTM(test,msg,...)
	#define ASSERTC(test,comp,val)
	#define ASSERTCM(test,comp,val,msg,...)
#endif

#endif // c++


//////////////////////////////////////////////////////////////////
// header
//////////////////////////////////////////////////////////////////

#ifndef CYRIL_MACRO_H
#define CYRIL_MACRO_H

/*
$ ulimit -c
$ ulimit -c unlimited
*/

// __func__ __FUNCTION__ __PRETTY_FUNCTION__
#if __STDC_VERSION__ >= 199901L || defined(__cplusplus)
  // "inline" is a keyword
#else
	#define inline // nothing
#endif

//////////////////////////////////////////////////////////////////
// includes
//////////////////////////////////////////////////////////////////


#include <stdio.h>

#include <math.h>
#include <stddef.h>
#include <stdlib.h>
#include <unistd.h>	
#include <malloc.h>
#include <string.h>
#include <signal.h>
#include <errno.h>


//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
// cpp compatibility
//////////////////////////////////////////////////////////////////

extern double CUMUL_ALLOC;
extern unsigned NB_ALLOC;
extern double PREV_CUMUL_ALLOC;

#ifndef _NO_ASSERT_
	#define new1_(type) new_(type, 1)
	#define new_(type, size) ({ type *newptr = (type *) malloc((size)*sizeof(type)); ASSERTM(newptr, "memory allocation failed for %dx%d=%d bytes", size, sizeof(type), size*sizeof(type)); newptr; })
/*	#define new_(type, size) ({ type *newptr = (type *) malloc((size)*sizeof(type)); ASSERTM(newptr, "memory allocation failed for %dx%d=%d bytes", size, sizeof(type), size*sizeof(type)); \
		CUMUL_ALLOC += size*sizeof(type); NB_ALLOC++; if (CUMUL_ALLOC > PREV_CUMUL_ALLOC+1e6) { DEBUG("allocating %d bytes (%d MB) ; total %g bytes (%g MB) in %d allocations", size*sizeof(type), (size*sizeof(type))>>20, CUMUL_ALLOC, CUMUL_ALLOC/1024/1024, NB_ALLOC); PREV_CUMUL_ALLOC = CUMUL_ALLOC; } \
		newptr; })
*/	#define delete_(ptr) { free(ptr); ptr = NULL; }
	#define renew_(ptr, type, size) ({ type *newptr = (type *) realloc(ptr, (size)*sizeof(type)); ASSERTM(newptr, "size %d", size); newptr; })
#else
	#define new1_(type) (type *) malloc(sizeof(type))
	#define new_(type, size) (type *) malloc((size)*sizeof(type))
	#define delete_(ptr) { free(ptr); }
	#define renew_(ptr, type, size) (type *) realloc(ptr, (size)*sizeof(type))
#endif

#include <stdbool.h>

//////////////////////////////////////////////////////////////////
// messages part2
//////////////////////////////////////////////////////////////////

#define BLANK_S 200
extern char BLANK[BLANK_S];
#define FILL_BLANK(size) (BLANK + BLANK_S-1 - min_(BLANK_S-1,max_(0,size)))
#define TAB_S 100
extern char TAB[TAB_S];
#define FILL_TAB(size) (TAB + TAB_S-1 - min(TAB_S-1,max(0,size)))


extern int INFO_INDENT;
extern int FN_SIZE;
extern FILE *LOG_FILE;
extern int STOP_PROGRAM;

#define teeprintf(format, ...) { fprintf(stderr, format, ## __VA_ARGS__); if (LOG_FILE) fprintf(LOG_FILE, format, ## __VA_ARGS__); }



//////////////////////////////////////////////////////////////////
// colors
//////////////////////////////////////////////////////////////////

/*
\033[
param;param;param
m

intensity 	0 	1 	2 	3 	4 	5 	6 	7 	9
Normal 	Black 	Red 	Green 	Yellow 	Blue 	Magenta 	Cyan 	White 	reset
Bright 	Black 	Red 	Green 	Yellow 	Blue 	Magenta 	Cyan 	White

SGR (Select Graphic Rendition) parameters Code 	Effect 	Note
0 	Reset / Normal 	all attributes off
1 	Intensity: Bold 	
2 	Intensity: Faint 	not widely supported
3 	Italic: on 	not widely supported. Sometimes treated as inverse.
4 	Underline: Single 	not widely supported
5 	Blink: Slow 	less than 150 per minute
6 	Blink: Rapid 	MS-DOS ANSI.SYS; 150 per minute or more
7 	Image: Negative 	inverse or reverse; swap foreground and background
8 	Conceal 	not widely supported
21 	Underline: Double 	
22 	Intensity: Normal 	not bold and not faint
24 	Underline: None 	
25 	Blink: off 	
27 	Image: Positive 	
28 	Reveal 	conceal off
3x 	Set foreground color, normal intensity 	
4x 	Set background color, normal intensity 	
9x 	Set foreground color, high intensity 	
10x 	set background color, high intensity 	

*/

#define clNORMAL "\033[1;00m"
#define clBWHITE "\033[1;29m"
#define clWHITE "\033[0;29m"
#define clBGRAY "\033[1;30m"
#define clGRAY "\033[0;30m"
#define clBRED "\033[1;31m"
#define clRED "\033[0;31m"
#define clBGREEN "\033[1;32m" 
#define clGREEN "\033[0;32m"
#define clBYELLOW "\033[1;33m"
#define clYELLOW "\033[0;33m"
#define clBBLUE "\033[1;34m"
#define clBLUE "\033[0;34m"
#define clBPURPLE "\033[1;35m"
#define clPURPLE "\033[0;35m"
#define clBCYAN "\033[1;36m"
#define clCYAN "\033[0;36m"


//////////////////////////////////////////////////////////////////
// math constants
//////////////////////////////////////////////////////////////////

#define PI M_PI


//////////////////////////////////////////////////////////////////
// types
//////////////////////////////////////////////////////////////////
// void ____________() {}

#define DOUBLE_PRECISION 1e-15
#define FLOAT_PRECISION 1e-10

#ifdef _USE_DOUBLE_
	typedef double real;
	#define REAL_PRECISION DOUBLE_PRECISION
#else
	typedef float real;
	#define REAL_PRECISION FLOAT_PRECISION
#endif

#include <sys/types.h>
// some are already defined in opencv
#ifndef _CXCORE_TYPES_H_
//typedef unsigned char uchar;
//typedef long long int int64;
typedef unsigned long long int uint64;
#endif
/*
typedef unsigned char byte;
typedef unsigned short ushort;
typedef unsigned short word;
typedef unsigned int uint;
typedef unsigned int dword;
*/
typedef uint64 Date;

// used to cast function adress passed to pthread_create
// the function must take one pointer as parameter and return a pointer
// cast (threadfn)&pouet is equivalent to (void* (*)(void*))&pouet
typedef void* (*threadfn)(void*);


//////////////////////////////////////////////////////////////////
// short functions
//////////////////////////////////////////////////////////////////
// void ____________() {}

// use sprintf in C
/*template<typename T>
static inline std::string xtostr(const T x) { std::ostringstream oss; oss << x; return oss.str();  }*/

//#define strtoi(s) atoi(s)
//#define strtof(s) atof(s)
//#define strtod(s) atol(s)

//extern int power(int x, int n);

#ifdef __cplusplus

template<typename T> inline T sqr(T x) { return x*x; }
template<typename T> inline T abs(T x) { return (x<0 ? -x : x); }
template<typename T> inline T min(T a, T b) { return (a<b ? a : b); }
template<typename T> inline T max(T a, T b) { return (a>b ? a : b); }
template<typename T> inline int sign(T x) { return (x >= 0 ? 1 : -1); }

template<typename T> inline T power(T x, int n) { T res=1; while(n) { if(n%2) res*=x; n/=2; x*=x; } return x; }

template<typename T> inline T mod(T x, int n) { return (x >= 0 ? x-n*(int)(x/n) : x-n*(int)(x/n)+n); }

template<typename T> inline T avg_mod(T left, T right, int n) 
	{ left = mod(left, n); right = mod(right, n); return (left <= right ? (left+right)/2 : mod((left+right+n)/2,n)); }


#else 

#define power(x,n) ({ int xx=(x), nn=(n), res=1; while(nn) { if(nn%2) res*=xx; nn/=2; xx*=xx; } xx; })
//#define log(x,n) ({ int xx=(x), nn=(n), res=; while(nn) { if(nn%2) res*=xx; nn/=2; xx*=xx; } xx; })

/** prendre la valeur absolue */
#define abs_(x) ((x) > 0 ? (x) : -(x))
#define abs(x) ({ typeof(x) xx=(x); abs_(xx); })

/** mettre au carr */
#define sqr_(x) ((x)*(x))
#define sqr(x) ({ typeof(x) xx=(x); sqr_(xx); })

/** max et min */
#define max_(a,b) ((a)>(b)?(a):(b))
#define max3_(a,b,c) (max_(max_(a,b),c))

#define max(a,b) ({ typeof(a) aa=(a); typeof (b) bb=(b); max_(aa,bb); })

#define max3(a,b,c) ({ typeof(a) aa=(a); typeof(b) bb=(b); typeof(c) cc=(c); max_(max_(aa,bb),cc); })

#define min_(a,b) ((a)<(b)?(a):(b))
#define min3_(a,b,c) (min_(min_(a,b),c))
#define min(a,b) ({ typeof(a) aa=(a); typeof(b) bb=(b); min_(aa,bb); })
#define min3(a,b,c) ({ typeof(a) aa=(a); typeof(b) bb=(b); typeof(c) cc=(c); min_(min_(aa,bb),cc); })

/** signe */
#define sign(x) ((x) < 0 ? -1 : +1)

#endif

/** V?ifie si x appartient ?l'intervalle [a,b], sans obligation de mettre a <= b */
#define isIn_(x,a,b) ((a)<(b) ? ((x)>=(a) && (x)<=(b)) : ((x)<=(a) && (x)>=(b)))
#define isIn(x,a,b) ({ typeof(x) xx=(x); typeof(a) aa=(a); typeof(b) bb=(b); isIn_(xx,aa,bb); })


// use this macro with caution because it's not side-effect proof : a and b are evaluated two times
#define myswap_(a,b) ({ typeof(a) tmp=(a); a=(b); b=tmp; })
#define myswap(a,b) ({ typeof(a) *aa=&(a), *bb=&(b); typeof(a) tmp=*aa; *aa=*bb; *bb=tmp; })


/** round to the nearest integer ; if you know the sign of x and you care about performance, use roundp or roundn */
#define roundp(x) ((int)((x)+0.5))
#define roundn(x) ((int)((x)-0.5))
#define round_(x) ((x)>=0 ? roundp(x) : roundn(x))
#define round(x) ({ typeof(x) xx=(x); round_(xx); })
/** truncate with tolerance ; eg float r = 2./3.; int rr = (int)(r*3); could give 1 or 2, and it can depend on optimization flags. int rr = trunc(r*3) will always give 2 */
#define truncp(x) ((int)((x)+1e-5))
#define truncn(x) ((int)((x)-1e-5))
#define trunc_(x) ((x)>=0 ? truncp(x) : truncn(x))
#define trunc(x) ({ typeof(x) xx=(x); trunc_(xx); })


/** nombres alatoires (gnrateur initialis dans la fonction d'init de la lib) */
#define random(n) (rand()%(n)) // returns a number between 0 and n-1
// oups ! rounding problem with the following : ((int)(((RAND_MAX-1)*(real)(n))/RAND_MAX)) == n !!!
//#define random(n) ((int)((rand()*(real)(n))/RAND_MAX)) // returns a number between 0 and n-1 ; because low order bits can be less random than high order bits
#define random2_(p,n) (random((n)-(p)+1)+(p)) // returns a number between p and n
#define random2(p,n) ({ typeof(p) pp=p; typeof(n) nn=n; random2_(pp,nn); })
#define randomr(r) (((real)(r)*(real)rand())/(((real)RAND_MAX)+1.))

#define mod_(x,n) ((x)-(n)*floor((x)/(n)))
//#define mod(x,n)  ({ typeof(x) xx=(x); typeof(n) nn=(n); mod_(xx,nn); })
#define mod2_(x,a,b) (mod_((x)-(a),(b)-(a))+(a))
#define mod2(x,a,b) ({ typeof(x) aa=(a); (mod((x)-aa,(b)-aa)+aa); })


/*
soft_eq : equal =
soft_ne : not equal !=
soft_lt : less than <
soft_le : less or equal <=
soft_gt : greater than >
soft_ge : greater or equal >=
*/

#ifdef _DOUBLE_COMP_
	#define soft_eq_(x,a,p) (abs((x)-(a)) <=(p))
	#define soft_ne_(x,a,p) (abs((x)-(a)) > (p))
	#define soft_lt_(x,a,p) ((x) < ((a)-(p)))
	#define soft_le_(x,a,p) ((x) <=((a)+(p)))
	#define soft_gt_(x,a,p) ((x) > ((a)+(p)))
	#define soft_ge_(x,a,p) ((x) >=((a)-(p)))
#else
	#define soft_eq_(x,a,p) ((x) == (a))
	#define soft_ne_(x,a,p) ((x) != (a))
	#define soft_lt_(x,a,p) ((x) <  (a))
	#define soft_le_(x,a,p) ((x) <= (a))
	#define soft_gt_(x,a,p) ((x) >  (a))
	#define soft_ge_(x,a,p) ((x) >= (a))
#endif

#define soft_eq(x,a) soft_eq_((x),(a),REAL_PRECISION)
#define soft_ne(x,a) soft_ne_((x),(a),REAL_PRECISION)
#define soft_lt(x,a) soft_lt_((x),(a),REAL_PRECISION)
#define soft_le(x,a) soft_le_((x),(a),REAL_PRECISION)
#define soft_gt(x,a) soft_gt_((x),(a),REAL_PRECISION)
#define soft_ge(x,a) soft_ge_((x),(a),REAL_PRECISION)


//////////////////////////////////////////////////////////////////
// initialisation programme
//////////////////////////////////////////////////////////////////
// void ____________() {}

void initCyrilLib(unsigned int *seed, int filename_size, char *log);















#endif










