/********************************************
Personal library of Cyril Roussillon
http://crteknologies.free.fr
cyril42e@gmail.com
2006-2007
*********************************************/

#ifndef CYRIL_STRUCT_H
#define CYRIL_STRUCT_H


//////////////////////////////////////////////////////////////////
// queue
//////////////////////////////////////////////////////////////////


typedef struct 
{
	void **elements;
	int size;
	int head; // first = o prendre, last = o ajouter (donc first = last => vide)
} Stack;

extern void allocStack(Stack *s, int size);
extern void freeStack(Stack *s);
extern bool isStackEmpty(Stack *s);
extern bool isStackFull(Stack *s);
extern int lengthStack(Stack *s);
extern void* popStack(Stack *s);
extern void pushStack(Stack *s, void *elt);
extern void flushStack(Stack *s);

/*
In fact a Queue and a Stack have almost the same structure, so that it is possible to add
both queue operations and stack operations to the same structure. Then it can be useful
to be able to add or retrieve elements at both ends
*/
typedef struct 
{
	void **elements;
	int size;
	int first, last; // first = o prendre, last = o ajouter (donc first = last => vide)
} Queue;

extern void allocQueue(Queue *q, int size);
extern void reallocQueue(Queue *q, int size);
extern void freeQueue(Queue *q);
extern bool isQueueEmpty(Queue *q);
extern bool isQueueFull(Queue *q);
extern int lengthQueue(Queue *q);
extern void* takeQueue(Queue *q);
extern void* popQueue(Queue *q);
extern void pushQueue(Queue *q, void *elt);
extern void insertQueue(Queue *q, void *elt);
extern void flushQueue(Queue *q);


//////////////////////////////////////////////////////////////////
// string parsing
//////////////////////////////////////////////////////////////////
// void ____________() {}


typedef enum NativeType_ { ntPtr = 0, ntChar, ntUChar, ntBool, ntShort, ntUShort, 
	ntInt, ntUInt, ntLInt, ntULInt, ntLLInt, ntULLInt, ntFloat, ntDouble, ntLDouble } NativeType;
	
int parserGetCount(char *s, char sep);
int parserExtract(char *s, char sep, void *data, int count, NativeType type);



//////////////////////////////////////////////////////////////////
// Couple
//////////////////////////////////////////////////////////////////
// void ____________() {}


typedef struct Couple_
{
	union
	{
		int X, x; int W, w; int U, u; int L, l; /// abscisse, width (largeur)
		int col; /// colonne
		int X1, x1; int Y1, y1; /// couples qcq
		int min; /// minimum
	};
	union
	{
		int Y, y; int H, h; int V,v; int R, r; /// ordonne, height (hauteur)
		int row; /// ligne
		int X2, x2; int Y2, y2; /// couples qcq
		int max; /// maximum
	};
} Couple;

typedef struct CoupleW_
{
	union
	{
		int X, x; int W, w; int U, u; int L, l; /// abscisse, width (largeur)
		int col; /// colonne
		int X1, x1; int Y1, y1; /// couples qcq
		int min; /// minimum
	};
	union
	{
		int Y, y; int H, h; int V,v; int R, r; /// ordonne, height (hauteur)
		int row; /// ligne
		int X2, x2; int Y2, y2; /// couples qcq
		int max; /// maximum
	};
	real a;
} CoupleW;

/** Rect */	
typedef struct Rect_
{
	union { int x,x1; }; /// x et x1 sont juste deux noms pour une mme zone mmoire
	union { int w,x2; };
	union { int y,y1; };
	union { int h,y2; };
} Rect;

/** RectW */
typedef struct RectW_
{
	union { int x,x1; }; /// x et x1 sont juste deux noms pour une mme zone mmoire
	union { int y,y1; };
	union { int w,x2; };
	union { int h,y2; };
	real a;
} RectW;

#endif


